const express = require("express");
const app = express();
const SerialPort = require("serialport");

var Readline = require("@serialport/parser-readline");

var feedback = "";
var busy = false;
var port;
var parser;
var arduinoport = "";


function initPort(portName, baudRate, callback) {
  console.log("Starting serial communication stuff")

  port = new SerialPort(portName, { baudRate: baudRate });
  parser = port.pipe(new Readline({ delimiter: "\n" }));

  // Read the port data
  port.on("open", function () {
    console.log("Serial port open");
  });

  parser.on("data", data => {
    feedback = data.substring(0, 2);
    console.log("Received: " + data);
  });

  console.log("port: " + portName + "; baud rate: " + baudRate);

  callback();
}


function getConnectedArduino(callback) {
  if (process.argv[2]) {
    arduinoport = process.argv[2];
    callback();
  } else {

    SerialPort.list().then(ports => {

      var allports = ports.length;
      var count = 0;
      var done = false

      ports.forEach(function (port) {
        count += 1;
        pm = port['manufacturer'];
        if (typeof pm !== 'undefined' && (pm.includes('arduino') || pm.includes('wch') || pm.includes('FTDI'))) {
          arduinoport = port.path.toString();
          callback();
          done = true;
        }
        if (count === allports && done === false) {
          console.log('cant find arduino')
        }
      });
    });

  }
}


function waitForFeedback(code, callback) {
  setTimeout(function () {
    //console.log("Waiting for feedback; " + "a"+feedback+"a");
    if (feedback == code) {
      callback();
      feedback = "";
      busy = false;
    } else {
      waitForFeedback(code, callback);
    }
  }, 100);
}

function waitForAvailable(callback) {
  setTimeout(function () {
    if (!busy) {
      busy = true;
      callback();
    } else {
      waitForAvailable(callback);
    }
  }, 100);
}


// Headers
app.use(function (req, res, next) {
  res.setHeader("Access-Control-Allow-Origin", "*");
  // console.log(req.url);

  if (req.path.includes("setport")) {
    next();
    return;
  }

  waitForAvailable(function () {
    //next();
    console.log(req.method);
    if (req.method == "GET"){
      var cmdStr = req.path.substr(1);
      port.write(cmdStr, function (err) {
        if (err) {
          res.send("Communication error");
          console.log("Error on write: ", err.message);
          return;
        }
        console.log("Message written: " + cmdStr);
        waitForFeedback("OK", function () {
          res.send("Task completed");
        });
      });
    }
    else{
      res.send("Received NON-GET request");
      console.log("Received NON-GET request");
      busy = false;
    }
  });
});

app.get("/", function (req, res) {
  res.send("Hello");
});

// Change port and baud rate from SNAP
app.get("/setport/:port/:baud", function (req, res) {
  var p = req.params.port;
  var b = req.params.baud;
  console.log("Connecting to serial port: " + p + " with baud rate: " + b);

  // Close old connection
  port.close(function (err) {
    console.log('port closed');
    if (err) {
      console.log(err);
      //res.send("Error closing the connection");
    }
    // Init new connection
    initPort(p, parseInt(b), function () {
      res.send("Connected to serial port: " + p + " with baud rate: " + b);
    });
  });
});

// app.get("/high", function (req, res) {
//   port.write("high", function (err) {
//     if (err) {
//       return console.log("Error on write: ", err.message);
//     }
//     console.log("Message written");
//     waitForFeedback("OK", function () {
//       res.send("Setted led high");
//     });
//   });
// });

// app.get("/low", function (req, res) {
//   port.write("low", function (err) {
//     if (err) {
//       return console.log("Error on write: ", err.message);
//     }
//     console.log("Message written");
//     waitForFeedback("OK", function () {
//       res.send("Setted led low");
//     });
//   });
// });

// app.get("/forward/:steps", function (req, res) {
//   var steps = req.params.steps;
//   port.write("forward/" + steps, function (err) {
//     if (err) {
//       return console.log("Error on write: ", err.message);
//     }
//     console.log("Message written");
//     waitForFeedback("OK", function () {
//       res.send("Moving forward");
//     });
//   });
// });

var server = app.listen(3002, function () {
  console.log("GeomBot server running on port 3002!");

  getConnectedArduino(function () {
    initPort(arduinoport, 9600, function () {
      console.log("Connection initialized");
    });
  });
});

process.on('exit', (code) => {
  server.close();
  console.log(`About to exit with code: ${code}`);
});
